/*
 *  TOPPERS BASE PLATFORM MIDDLEWARE
 * 
 *  Copyright (C) 2017-2024 by TOPPERS PROJECT
 *                             Educational Working Group.
 * 
 *  上記著作権者は，以下の(1)～(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 * 
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 * 
 *  @(#) $Id: protocal.c 698 2024-03-22 17:55:40Z roi $
 */
/*
 *  BLE MOTOR制御 Middleware
 */

#ifndef _PROTOCOL_H_
#define _PROTOCOL_H_

#define EVENT_IDOL              0
#define EVENT_SEND_RESPONSE     1
#define EVENT_WRITE_FLASH       2
#define EVENT_MOTOR_EVENT       3
#define EVENT_IDOL_ERROR        10

#define DEVICE_BLE              0
#define DEVICE_UART             1

#define NUM_MOTORS              2
#define MOTOR1_ID               0
#define MOTOR2_ID               1
#ifndef NUM_STBYS
#define NUM_STBYS               NUM_MOTORS
#endif

#define DUTY_PERIOD             1000L		/* 1000Hz **/
#define DEFAULT_BIAS            10000		/* 1.0 */
#define DEFAULT_MOTOR_TIME      600			/* 600ms */
#define DEFAULT_MIN_USS_INT     1500		/* 1.5sec */
#define DEFAULT_MIN_USS_TIME    100			/* 100ms */
#define DEAFULT_MIN_USS_REPORT  20			/* 20mm */

#define EEPROM_ADDR             0x50
#define EEPROM_INFO_ADDRESS     0
#define EEPROM_ADDRESS_LEN_BYTES 2
#define EEPROM_WRITE_MAX_BYTES  32
#define US_SENSOR_ADDR          0x2C
#define US_SENSOR_START_CMD     0x33
#define US_SENSOR_DATAH_CMD     0x0F
#define US_SENSOR_DATAL_CMD     0x0E

#define EEPROM_INVALID_LENGTH   0x8210
#define EEPROM_INVALID_ADDR     0x8211

#define EVENT_MOTOR             0x00000001
#define EVENT_USS               0x00000002

#define BLECOM_ERROR_EEPROM     1
#define BLECOM_ERROR_TWI        2
#define BLECOM_ERROR_USS        3
#define BLECOM_ERROR_TIME       4
#define BLECOM_ERROR_PWM        5
#define BLECOM_ERROR_BLE        6
#define BLECOM_ERROR_ADC        7
#define BLECOM_ERROR_SYS        8

/*
 *  FLASH-ROM用の構造体定義
 */
typedef struct _EEPROM {
	uint8_t   magic[2];			/* マジックコード */
	char      devicename[32];	/* デバイス名 */
	uint8_t   t_mselect;		/* TC78H653FTG-MOTOR-SELECT */
	uint8_t   dummy1;
	uint8_t   t_m_pulse[NUM_MOTORS];	/* motor PULSE */
	uint16_t  t_m_bias[NUM_MOTORS];		/* motor bias */
	uint16_t  t_m_time;					/* motor wait time */
	uint16_t  u_s_time;					/* us sensor wait time */
	uint16_t  u_s_interval;				/* us sensor interval time */
	uint16_t  u_s_report_des;			/* us sensor report destance */
	char      dummy[14];
} EEPROM_t;


extern EEPROM_t *get_eeprom(void);
extern uint8_t get_syserror(void);
extern uint32_t get_systime(void);
extern uint32_t get_batterylevel(void);
extern void motor_duty_set(uint8_t id, int32_t value1, int32_t value2);
extern void motor_stby_set(uint8_t id, uint8_t mode);
extern uint32_t twi_write(uint8_t addr, uint8_t const *pdata, uint8_t len, int no_stop);
extern uint32_t twi_read(uint8_t addr, uint8_t *pdata, size_t size);
extern void uss_event_request(uint32_t interval, uint32_t time);

extern int dispatch(char *res, char *cmd, uint16_t *plen);
extern uint32_t eeprom_write(uint16_t addr, uint8_t const *pdata, uint8_t size);
extern uint32_t eeprom_read(uint16_t addr, uint8_t * pdata, uint16_t size);
extern void motor_init(void);
extern uint32_t motor_event(void);
extern int uss_start(void);
extern int uss_get_result(uint8_t uss_value);
extern int uss_make_event(char *res, uint32_t uss_value1, uint32_t uss_value2, uint16_t *plen);

#endif	/* _PROTOCOL_H_ */

